"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const os = tslib_1.__importStar(require("os"));
const electron_1 = require("electron");
const electron_log_1 = require("electron-log");
const fu = tslib_1.__importStar(require("../common/file-utils"));
const dev_1 = require("./dev");
const backend_1 = require("../ipc-bridge/backend");
const optimize_1 = tslib_1.__importDefault(require("./optimize"));
const save_1 = require("./save");
const menu_1 = tslib_1.__importDefault(require("./menu"));
const menuActions = tslib_1.__importStar(require("./menu-actions"));
const imagemagick_1 = require("./imagemagick");
class App {
    constructor() {
        this.windows = [];
        this.menu = menu_1.default;
        this.ready = new Promise((resolve) => {
            electron_1.ipcMain.once("READY" /* READY */, resolve);
        });
        this.onOtherInstance = (event, argv) => {
            const win = this.getMainWindow();
            win && win.focus();
            this.receiveFiles(argv.slice(1));
        };
        this.handleIpcFileSave = async (event, images, type) => {
            const saveToDir = async (dirname) => {
                await save_1.saveFiles(images, type, dirname);
                event.sender.send("SAVED" /* SAVED */);
            };
            const mainWindow = this.getMainWindow();
            if (type === "NEW_DIR" /* NEW_DIR */) {
                const { filePaths } = await electron_1.dialog.showOpenDialog(mainWindow, {
                    title: 'Save files',
                    properties: ['openDirectory', 'createDirectory'],
                });
                if (!filePaths || !filePaths.length)
                    return;
                const dirpath = filePaths[0];
                await saveToDir(dirpath);
                electron_1.shell.openItem(dirpath);
            }
            else if (type === "SAVE_AS" /* SAVE_AS */) {
                const image = images[0];
                const { filePath } = await electron_1.dialog.showSaveDialog(mainWindow, {
                    title: 'Save files',
                    defaultPath: fu.reext(image.originalName, image.ext),
                    filters: [{
                            name: 'Images',
                            extensions: [image.ext],
                        }],
                });
                if (filePath) {
                    await save_1.saveFile(images[0], filePath);
                    event.sender.send("SAVED" /* SAVED */);
                }
            }
            else {
                saveToDir();
            }
        };
    }
    start() {
        const gotTheLock = electron_1.app.requestSingleInstanceLock();
        if (!gotTheLock) {
            electron_1.app.quit();
        }
        else {
            electron_1.app.on('window-all-closed', () => {
                electron_1.app.quit();
            });
            electron_1.app.on('second-instance', this.onOtherInstance);
            this.createWindow();
            this.menu.render();
            this.listenIpc();
            this.listenMenu();
        }
    }
    getMainWindow() {
        const id = this.windows[0];
        if (id != null) {
            return electron_1.BrowserWindow.fromId(id);
        }
    }
    createWindow() {
        const baseWidth = 800;
        const win = new electron_1.BrowserWindow({
            width: baseWidth + (os.platform() === 'darwin' ? 15 : 34),
            height: 600,
            minWidth: 540,
            // titleBarStyle: 'hidden',
            webPreferences: {
                nodeIntegration: true,
                webSecurity: false,
            },
        });
        const { id } = win;
        electron_log_1.log(dev_1.url);
        win.loadURL(dev_1.url).then(() => {
            electron_log_1.log('url loaded:', dev_1.url);
        }, (err) => {
            electron_log_1.log('url failed:', dev_1.url, err);
        });
        win.on('closed', () => {
            const index = this.windows.indexOf(id);
            if (index > -1) {
                this.windows.splice(index, 1);
            }
        });
        win.webContents.on('will-navigate', e => {
            e.preventDefault();
        });
        if (process.env.NODE_ENV === 'development') {
            win.webContents.openDevTools();
        }
        this.windows.push(id);
    }
    async receiveFiles(filePaths) {
        const files = await fu.flattenFiles(filePaths);
        const dests = (await fu.saveFilesTmp(files)).filter(x => x);
        await this.ready;
        const win = this.getMainWindow();
        win && win.webContents.send("FILE_SELECTED" /* FILE_SELECTED */, dests);
    }
    listenMenu() {
        this.menu.on('save', (type) => {
            const win = this.getMainWindow();
            win && win.webContents.send("SAVE" /* SAVE */, type);
        });
    }
    listenIpc() {
        backend_1.listenIpc("OPTIMIZE" /* OPTIMIZE */, ({ image, exportExt, options }) => optimize_1.default(image, options));
        backend_1.listenIpc("DETECT_IMAGEMAGICK" /* DETECT_IMAGEMAGICK */, imagemagick_1.detectImageMagick);
        electron_1.ipcMain.on("FILE_SELECT" /* FILE_SELECT */, () => {
            menuActions.open();
        });
        electron_1.ipcMain.on("FILE_ADD" /* FILE_ADD */, (event, files) => {
            this.receiveFiles(files);
        });
        electron_1.ipcMain.on("SAVE" /* SAVE */, this.handleIpcFileSave);
        electron_1.ipcMain.on("SYNC" /* SYNC */, (event, state) => {
            const { menu } = this;
            menu.taskCount = state.taskCount;
            menu.aloneMode = state.aloneMode;
            menu.render();
        });
    }
}
exports.default = new App();
