"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const os = tslib_1.__importStar(require("os"));
const fs = tslib_1.__importStar(require("fs-extra"));
const path = tslib_1.__importStar(require("path"));
const crypto = tslib_1.__importStar(require("crypto"));
const file_type_1 = tslib_1.__importDefault(require("file-type"));
const read_chunk_1 = tslib_1.__importDefault(require("read-chunk"));
const raw_body_1 = tslib_1.__importDefault(require("raw-body"));
const constants_1 = require("../common/constants");
const electron_log_1 = tslib_1.__importDefault(require("electron-log"));
exports.tmpdir = path.resolve(os.tmpdir(), 'imageOptimizer');
exports.isSupportedExt = (type) => {
    return type in constants_1.SupportedExt;
};
exports.cleanTmpdir = () => fs.emptyDirSync(exports.tmpdir);
const takeSubHash = (hash) => hash.replace(/[/+=]/g, '').slice(0, 32);
function md5(text) {
    return crypto.createHash('md5').update(text).digest('hex');
}
exports.md5 = md5;
function getSize(filePath) {
    return fs.stat(filePath).then(stats => stats.size);
}
exports.getSize = getSize;
async function fileMD5(filePath) {
    const hash = crypto.createHash('md5');
    fs.createReadStream(filePath).pipe(hash);
    return raw_body_1.default(hash, {
        encoding: 'hex',
    });
}
exports.fileMD5 = fileMD5;
async function imageType(file) {
    if (typeof file === 'string') {
        file = await read_chunk_1.default(file, 0, 12);
    }
    return file_type_1.default(file);
}
exports.imageType = imageType;
exports.getFilePath = (image) => path.resolve(exports.tmpdir, image.id + '.' + image.ext);
exports.saveFilesTmp = (files) => {
    return Promise.all(files.map(async (file) => {
        const type = await imageType(file);
        const ext = type && type.ext;
        if (!ext || !exports.isSupportedExt(ext))
            return;
        const id = md5(file) + await fileMD5(file);
        const size = await getSize(file);
        const descriptor = {
            size,
            id,
            ext,
            originalName: file,
        };
        const dest = exports.getFilePath(descriptor);
        descriptor.url = 'file://' + dest;
        await fs.copy(file, dest);
        return descriptor;
    }));
};
/**
 * get a unoccupied file path by an orignial path.
 * example: `/path/to/a.txt` to `/path/to/a(1).txt`
 * @param filePath - original file path
 */
exports.unoccupiedFile = (filePath, index = 0) => {
    const accessPath = index
        ? filePath.replace(/(\.\w+)?$/, `(${index})$1`)
        : filePath;
    return fs.access(accessPath)
        .then(() => exports.unoccupiedFile(filePath, index + 1))
        .catch(() => accessPath);
};
/**
 * walk dir/file list to a flat files list
 */
exports.flattenFiles = async (filePaths) => {
    let list = [];
    for (const filePath of filePaths) {
        try {
            const stat = await fs.stat(filePath);
            if (stat.isFile()) {
                list.push(filePath);
            }
            else if (stat.isDirectory()) {
                const dirFileNames = await fs.readdir(filePath);
                const dirFiles = dirFileNames.map(name => path.resolve(filePath, name));
                const dirFlatFiles = await exports.flattenFiles(dirFiles);
                list = list.concat(dirFlatFiles);
            }
        }
        catch (e) {
            electron_log_1.default.error(`Failed to access file ${filePath}`);
        }
    }
    return list;
};
/**
 * 'path/to/image.png' + jpg -> 'path/to/image.jpg'
 * @param filename - 'path/to/image.png'
 * @param ext - jpg
 */
exports.reext = (filename, ext) => {
    return filename.replace(/(?:\.(\w+))?$/i, ($0, $1) => {
        $1 = $1.toLowerCase();
        // make sure `x.PNG` not be transformed to `x.png`
        if ($1 === ext) {
            return $0;
        }
        if ($1 in constants_1.SupportedExt) {
            return '.' + ext;
        }
        else {
            return $0 + '.' + ext;
        }
    });
};
