"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const dom_event_1 = require("../utils/dom-event");
const utils_1 = require("../../common/utils");
const RadioGroup_1 = tslib_1.__importDefault(require("./RadioGroup"));
const Icon_1 = tslib_1.__importDefault(require("./Icon"));
require("./ImageViewer.less");
const ZOOM_MIN = 1 / 8;
const ZOOM_MAX = 16;
const zoomCoop = utils_1.coop(ZOOM_MIN, ZOOM_MAX);
const roundZoom = (zoom) => {
    return Math.pow(2, Math.round(Math.log2(zoom)));
};
const floorZoom = (zoom) => {
    return Math.pow(2, Math.floor(Math.log2(zoom)));
};
const materials = [
    'gradient',
    'transparent',
    'white',
    'black',
    'red',
    'blue',
    'yellow',
];
class ImageViewer extends react_1.PureComponent {
    constructor(props) {
        super(props);
        this.prevScreenX = 0;
        this.prevScreenY = 0;
        this.dragging = false;
        this.handleImageLoad = () => {
            const { naturalWidth, naturalHeight } = this.image;
            this.setState({
                imageNaturalWidth: naturalWidth,
                imageNaturalHeight: naturalHeight,
                imageError: false,
            });
            if (!this.state.imageNaturalWidth) {
                this.setState({
                    zoom: this.initialZoom(),
                });
            }
        };
        this.handleImageError = () => {
            this.setState({
                imageError: true,
            });
        };
        this.handleMouseDown = (e) => {
            e.preventDefault();
            this.dragging = true;
            this.prevScreenX = e.screenX;
            this.prevScreenY = e.screenY;
        };
        this.handleMouseMove = (e) => {
            if (!this.dragging)
                return;
            const { screenX, screenY } = e;
            this.setState({
                x: this.state.x + screenX - this.prevScreenX,
                y: this.state.y + screenY - this.prevScreenY,
            });
            this.prevScreenX = screenX;
            this.prevScreenY = screenY;
        };
        this.handleMouseUp = (e) => {
            this.dragging = false;
        };
        this.handleWheel = (e) => {
            e.preventDefault();
            const wheelData = dom_event_1.consumWheelEvent(e);
            const { state } = this;
            if (wheelData.type === 'zoom') {
                const target = e.currentTarget;
                const mousePosition = dom_event_1.eventOffset(e.nativeEvent, target);
                const zoom = zoomCoop(state.zoom * wheelData.zoom);
                const mouseX = mousePosition.x - target.clientWidth / 2;
                const x = mouseX - (mouseX - state.x) / state.zoom * zoom;
                const mouseY = mousePosition.y - target.clientHeight / 2;
                const y = mouseY - (mouseY - state.y) / state.zoom * zoom;
                this.setState({
                    x, y, zoom,
                });
            }
            else {
                this.setState({
                    x: state.x - wheelData.x,
                    y: state.y - wheelData.y,
                });
            }
        };
        this.handleZoomOut = (e) => {
            e.stopPropagation();
            const { x, y, zoom } = this.state;
            this.setState({
                zoom: roundZoom(zoom * 2),
                x: 0,
                y: 0,
            });
        };
        this.handleZoomIn = (e) => {
            e.stopPropagation();
            const { x, y, zoom } = this.state;
            this.setState({
                zoom: roundZoom(zoom / 2),
                x: 0,
                y: 0,
            });
        };
        this.handleFocusCenter = (e) => {
            e.stopPropagation();
            this.setState({
                x: 0,
                y: 0,
                zoom: this.initialZoom(),
            });
        };
        this.handleMaterialChange = (value) => {
            this.setState({
                material: value,
            });
        };
        this.state = {
            imageNaturalWidth: 0,
            imageNaturalHeight: 0,
            zoom: 1,
            x: 0,
            y: 0,
            material: materials[0],
            imageError: false,
        };
    }
    componentDidMount() {
        document.addEventListener('mousemove', this.handleMouseMove);
        document.addEventListener('mouseup', this.handleMouseUp);
    }
    componentWillUnmount() {
        document.removeEventListener('mousemove', this.handleMouseMove);
        document.removeEventListener('mouseup', this.handleMouseUp);
    }
    imageSize() {
        let { width, height } = this.props;
        if (!(width && height)) {
            width = this.state.imageNaturalWidth;
            height = this.state.imageNaturalHeight;
        }
        return {
            width,
            height,
        };
    }
    initialZoom() {
        if (!this.image) {
            return 1;
        }
        const { naturalWidth, naturalHeight } = this.image;
        const { clientWidth, clientHeight } = this.backdrop;
        return floorZoom(Math.min(clientWidth / naturalWidth, clientHeight / naturalHeight) * 0.9);
    }
    renderMaterialItem(material) {
        return react_1.default.createElement("div", { className: classnames_1.default('material-cube', '-' + material) });
    }
    renderContent() {
        const { zoom, x, y, imageError, } = this.state;
        if (imageError) {
            return react_1.default.createElement("div", { className: "image-fail" }, "FAILED");
        }
        if (this.props.src) {
            return (react_1.default.createElement("img", { className: "image -transition", src: this.props.src, onLoad: this.handleImageLoad, onError: this.handleImageError, ref: el => { this.image = el; }, style: {
                    transform: `translate(${x}px, ${y}px) scale(${zoom})`,
                } }));
        }
        else {
            this.image = undefined;
        }
        return null;
    }
    render() {
        const { zoom, x, y, material, } = this.state;
        const { width, height } = this.imageSize();
        return (react_1.default.createElement("div", { className: "backdrop", onWheel: this.handleWheel, ref: el => { this.backdrop = el; } },
            react_1.default.createElement("div", { className: classnames_1.default('material-wall', '-' + material) }),
            react_1.default.createElement("div", { className: "image-wrapper", onMouseDown: this.handleMouseDown }, this.renderContent()),
            react_1.default.createElement("div", { className: "zoom-control" },
                react_1.default.createElement("button", { className: "paper", onClick: this.handleFocusCenter },
                    react_1.default.createElement(Icon_1.default, { name: "focus" })),
                react_1.default.createElement("button", { className: "paper", onClick: this.handleZoomOut, disabled: zoom >= ZOOM_MAX }, "+"),
                react_1.default.createElement("button", { className: "paper", onClick: this.handleZoomIn, disabled: zoom <= ZOOM_MIN }, "-")),
            react_1.default.createElement(RadioGroup_1.default, { className: "material-list", data: materials, value: material, renderItem: this.renderMaterialItem, onChange: this.handleMaterialChange })));
    }
}
exports.default = ImageViewer;
