"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const redux_actions_1 = require("redux-actions");
const storage = tslib_1.__importStar(require("./storage"));
const constants_1 = require("../../common/constants");
exports.createOptimizeOptions = (ext) => {
    const optimizeOptions = {
        exportExt: ext,
    };
    switch (ext) {
        case constants_1.SupportedExt.jpg:
        case constants_1.SupportedExt.webp:
            Object.assign(optimizeOptions, {
                quality: 80,
            });
            break;
        case constants_1.SupportedExt.png:
            Object.assign(optimizeOptions, {
                color: 128,
            });
            break;
    }
    return optimizeOptions;
};
const savedOptions = storage.getOptions();
function updateTaskItem(state, id, partial) {
    return updateTaskList(state, tasks => {
        const index = tasks.findIndex(task => task.id === id);
        if (index === -1)
            return tasks;
        return [
            ...tasks.slice(0, index),
            {
                ...tasks[index],
                ...partial,
            },
            ...tasks.slice(index + 1),
        ];
    });
}
function updateTaskList(state, updater) {
    return {
        ...state,
        tasks: updater(state.tasks),
    };
}
function updateGlobals(state, updater) {
    return {
        ...state,
        globals: updater(state.globals),
    };
}
function updateGlobalsPartial(state, updater) {
    return updateGlobals(state, globals => ({
        ...globals,
        ...updater,
    }));
}
function getInitialTaskOptions(exportExt, defaultOptions) {
    return {
        ...(defaultOptions[exportExt] || exports.createOptimizeOptions(exportExt)),
        exportExt,
    };
}
exports.default = redux_actions_1.handleActions({
    ["TASK_ADD" /* TASK_ADD */](state, action) {
        const { defaultOptions } = state.globals;
        return updateTaskList(state, tasks => [
            ...tasks,
            ...action.payload
                .filter(image => !tasks.some(task => task.id === image.id))
                .map(image => {
                const exportExt = (defaultOptions[image.ext] &&
                    defaultOptions[image.ext].exportExt ||
                    image.ext);
                return {
                    id: image.id,
                    image,
                    options: getInitialTaskOptions(exportExt, defaultOptions),
                    status: "PENDING" /* PENDING */,
                };
            }),
        ]);
    },
    ["TASK_DELETE" /* TASK_DELETE */](state, action) {
        return updateTaskList(state, tasks => tasks.filter(task => !action.payload.some(id => id === task.id)));
    },
    ["TASK_CLEAR" /* TASK_CLEAR */](state, action) {
        return updateTaskList(state, () => []);
    },
    ["TASK_UPDATE_OPTIONS" /* TASK_UPDATE_OPTIONS */](state, action) {
        const { id, options } = action.payload;
        return updateTaskItem(state, id, {
            options,
            status: "PENDING" /* PENDING */,
        });
    },
    ["TASK_UPDATE_EXPORT" /* TASK_UPDATE_EXPORT */](state, action) {
        const { id, exportExt } = action.payload;
        const { defaultOptions } = state.globals;
        return updateTaskItem(state, id, {
            options: getInitialTaskOptions(exportExt, defaultOptions),
            status: "PENDING" /* PENDING */,
        });
    },
    ["TASK_OPTIMIZE_START" /* TASK_OPTIMIZE_START */](state, action) {
        const id = action.payload;
        return updateTaskItem(state, id, {
            status: "PROCESSING" /* PROCESSING */,
        });
    },
    ["TASK_OPTIMIZE_SUCCESS" /* TASK_OPTIMIZE_SUCCESS */](state, action) {
        const { id, optimized } = action.payload;
        return updateTaskItem(state, id, {
            optimized,
            status: "DONE" /* DONE */,
        });
    },
    ["TASK_OPTIMIZE_FAIL" /* TASK_OPTIMIZE_FAIL */](state, action) {
        const id = action.payload;
        return updateTaskItem(state, id, {
            status: "FAIL" /* FAIL */,
        });
    },
    ["OPTIONS_APPLY" /* OPTIONS_APPLY */](state) {
        const { defaultOptions } = state.globals;
        return updateTaskList(state, list => list.map(item => {
            const exportExt = defaultOptions[item.image.ext].exportExt;
            return {
                ...item,
                options: getInitialTaskOptions(exportExt, defaultOptions),
                status: "PENDING" /* PENDING */,
            };
        }));
    },
    ["TASK_SELECTED_ID_UPDATE" /* TASK_SELECTED_ID_UPDATE */](state, action) {
        return updateGlobalsPartial(state, {
            activeId: action.payload,
        });
    },
    ["APP_UPDATABLE" /* APP_UPDATABLE */](state, action) {
        return updateGlobalsPartial(state, {
            updateInfo: action.payload,
        });
    },
    ["OPTIONS_VISIBLE_UPDATE" /* OPTIONS_VISIBLE_UPDATE */](state, action) {
        return updateGlobalsPartial(state, {
            optionsVisible: action.payload,
        });
    },
    ["DEFAULT_OPTIONS_UPDATE" /* DEFAULT_OPTIONS_UPDATE */](state, action) {
        const { ext, options } = action.payload;
        const defaultOptions = {
            ...state.globals.defaultOptions,
            [ext]: options,
        };
        /**
         * save to localStorage
         */
        storage.saveOptions({ defaultOptions });
        return updateGlobalsPartial(state, {
            defaultOptions,
        });
    },
    ["IMAGEMAGICK_CHECKED_UPDATE" /* IMAGEMAGICK_CHECKED_UPDATE */](state, action) {
        return updateGlobalsPartial(state, {
            imageMagickInstalled: action.payload,
        });
    },
}, {
    tasks: [],
    globals: {
        optionsVisible: false,
        imageMagickInstalled: false,
        defaultOptions: {
            png: exports.createOptimizeOptions(constants_1.SupportedExt.png),
            jpg: exports.createOptimizeOptions(constants_1.SupportedExt.jpg),
            webp: exports.createOptimizeOptions(constants_1.SupportedExt.webp),
        },
        ...savedOptions,
    },
});
